/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "pcaloptions.h"
#include<fstream>
#include<iostream>
#include<cstdio>
#include<cstdlib>
#include <libintl.h>
#include<errno.h>
#include "defaults.h"
#include "globals.h"

PCalOptions::PCalOptions() 
{}
    
PCalOptions::PCalOptions(Gtk::Window * mw):
mainWindow(mw)
{
    // get the user home path
    homepath = Glib::ustring(getenv("HOME"));
    if(homepath[homepath.length()-1] !='/') homepath+="/";

    // now load the configuration file
    // look for the local configuration first
    std::ifstream infile(Glib::ustring(homepath + LOCAL_CFG_FILE).c_str());
    if(!infile) {
        // there is no config file - create the new one with the default options
        resetToDefaults();
        saveLocalOptions();
    } else {
        // load parameters from the local config
        loadLocalOptions();
    }
}

PCalOptions& PCalOptions::operator=(const PCalOptions& src)
{
    homepath = src.homepath;
    firstDay = src.firstDay;
    toolbarStyle = src.toolbarStyle; 
    // new option placeholder    
    undefinedMonthBGColor = src.undefinedMonthBGColor;
    undefinedMonthFGColor = src.undefinedMonthFGColor;
    inactiveMonthBGColor = src.inactiveMonthBGColor;
    inactiveMonthFGColor = src.inactiveMonthFGColor;
    todayBGColor = src.todayBGColor;
    todayFGColor = src.todayFGColor;
    day1BGColor = src.day1BGColor;
    day1FGColor = src.day1FGColor;
    pday1BGColor = src.pday1BGColor;
    pday1FGColor = src.pday1FGColor;
    fertBGColor = src.fertBGColor;
    fertFGColor = src.fertFGColor;
    pfertBGColor = src.pfertBGColor;
    pfertFGColor = src.pfertFGColor;
    pregFGColor = src.pregFGColor;
    pregBGColor = src.pregBGColor;
    birthFGColor = src.birthFGColor;
    birthBGColor = src.birthBGColor;
    pbirthFGColor = src.pbirthFGColor;
    pbirthBGColor = src.pbirthBGColor;
    bufferDays = src.bufferDays;
    badFeelDaysBefore = src.badFeelDaysBefore;
    badFeelDaysAfter = src.badFeelDaysAfter;
    regularMinLength = src.regularMinLength;
    regularMaxLength = src.regularMaxLength;
    mainWindow = src.mainWindow;
    
    return *this;
}

PCalOptions::~PCalOptions()
{

}

void PCalOptions::saveLocalOptions()
{
    Glib::ustring filename = Glib::ustring(homepath + LOCAL_CFG_FILE);
    std::ofstream outfile(filename.c_str());
    if(!outfile) {
        Gtk::MessageDialog *edialog = new class Gtk::MessageDialog(
                              gettext("Can not save the configuration file: \n")+
                              filename,
		                      false,
                              Gtk::MESSAGE_ERROR,
                              Gtk::BUTTONS_OK,
                              true);
        edialog->set_transient_for(*mainWindow);
        edialog->run();
        delete edialog;
    } else {
        // save all configuration lines
        outfile<<CFG_HEADER;
        outfile<<"\nFirst Day="<<firstDay;
        outfile<<"\nToolbar Style="<<toolbarStyle;
        // new option placeholder
        outfile<<"\nUndefined Month BG="<<color_to_str(undefinedMonthBGColor);
        outfile<<"\nUndefined Month FG="<<color_to_str(undefinedMonthFGColor);
        outfile<<"\nInactive Month BG="<<color_to_str(inactiveMonthBGColor);
        outfile<<"\nInactive Month FG="<<color_to_str(inactiveMonthFGColor);
        outfile<<"\nToday BG="<<color_to_str(todayBGColor);
        outfile<<"\nToday FG="<<color_to_str(todayFGColor);
        outfile<<"\nDay 1 BG="<<color_to_str(day1BGColor);
        outfile<<"\nDay 1 FG="<<color_to_str(day1FGColor);
        outfile<<"\nPredicted Day 1 BG="<<color_to_str(pday1BGColor);
        outfile<<"\nPredicted Day 1 FG="<<color_to_str(pday1FGColor);
        outfile<<"\nFertile BG="<<color_to_str(fertBGColor);
        outfile<<"\nFertile FG="<<color_to_str(fertFGColor);
        outfile<<"\nPredicted Fertile BG="<<color_to_str(pfertBGColor);
        outfile<<"\nPredicted Fertile FG="<<color_to_str(pfertFGColor);
        outfile<<"\nPregnancy BG="<<color_to_str(pregBGColor);
        outfile<<"\nPregnancy FG="<<color_to_str(pregFGColor);
        outfile<<"\nBirth BG="<<color_to_str(birthBGColor);
        outfile<<"\nBirth FG="<<color_to_str(birthFGColor);
        outfile<<"\nPredicted Birth BG="<<color_to_str(pbirthBGColor);
        outfile<<"\nPredicted Birth FG="<<color_to_str(pbirthFGColor);
        outfile<<"\nBuffer Days="<<bufferDays;
        outfile<<"\nBad Feels Before="<<badFeelDaysBefore;
        outfile<<"\nBad Feels After="<<badFeelDaysAfter;
        outfile<<"\nReg. Min Length="<<regularMinLength;
        outfile<<"\nReg. Max Length="<<regularMaxLength;        
    }
}

void PCalOptions::loadLocalOptions()
{
    Glib::ustring filename(homepath + LOCAL_CFG_FILE);
    std::ifstream infile(filename.c_str());
    if(!infile) {
        Gtk::MessageDialog *edialog = new class Gtk::MessageDialog(
                              gettext("Can not load the configuration file: \n")+
                              filename,
		                      false,
                              Gtk::MESSAGE_ERROR,
                              Gtk::BUTTONS_OK,
                              true);
        edialog->set_transient_for(*mainWindow);
        edialog->run();
        delete edialog;
    } else {
        std::string buffer;
        Glib::ustring curr_line;
        while (!infile.eof()) {
            std::getline(infile,buffer,'\n');
            // back to UNICODE
            curr_line = Glib::ustring(buffer);
            if(curr_line.size()==0) continue;
            // trim the leading whitespace
            unsigned nwpos = curr_line.find_first_not_of(" \t");
            if(nwpos==std::string::npos) continue;
            else
            if(nwpos>0) curr_line = curr_line.substr(nwpos);
    
            // is it comment?
            if(curr_line[0]=='#') continue;
    
            // parse the line
            unsigned i = curr_line.find("=");
            // lines without '=' are ignored
            if(i!=std::string::npos) {
                setKey(curr_line.substr(0,i), curr_line.substr(i+1,std::string::npos));
            }
        } // while
    } // else
}
    
void PCalOptions::resetToDefaults()
{
    firstDay = DEFAULT_FIRST_DAY;
    toolbarStyle = DEFAULT_TOOLBAR_STYLE;
    // new option placeholder
    undefinedMonthBGColor = DEFAULT_UNDEFINED_MONTH_BG_COLOR;
    undefinedMonthFGColor = DEFAULT_UNDEFINED_MONTH_FG_COLOR;
    inactiveMonthBGColor = DEFAULT_INACTIVE_MONTH_BG_COLOR;
    inactiveMonthFGColor = DEFAULT_INACTIVE_MONTH_FG_COLOR;
    todayBGColor = DEFAULT_TODAY_BG_COLOR;
    todayFGColor = DEFAULT_TODAY_FG_COLOR;
    day1BGColor = DEFAULT_DAY1_BG_COLOR;
    day1FGColor = DEFAULT_DAY1_FG_COLOR;
    pday1BGColor = DEFAULT_PDAY1_BG_COLOR;
    pday1FGColor = DEFAULT_PDAY1_FG_COLOR;
    fertBGColor = DEFAULT_FERT_BG_COLOR;
    fertFGColor = DEFAULT_FERT_FG_COLOR;
    pfertBGColor = DEFAULT_PFERT_BG_COLOR;
    pfertFGColor = DEFAULT_PFERT_FG_COLOR;
    pregBGColor = DEFAULT_PREG_BG_COLOR;
    pregFGColor = DEFAULT_PREG_FG_COLOR;
    birthBGColor = DEFAULT_BIRTH_BG_COLOR;
    birthFGColor = DEFAULT_BIRTH_FG_COLOR;
    pbirthBGColor = DEFAULT_PBIRTH_BG_COLOR;
    pbirthFGColor = DEFAULT_PBIRTH_FG_COLOR;
    bufferDays = DEFAULT_BUFFER_DAYS;
    badFeelDaysBefore = DEFAULT_BAD_FEEL_DAYS_BEFORE;
    badFeelDaysAfter = DEFAULT_BAD_FEEL_DAYS_AFTER;
    regularMinLength = DEFAULT_REGULAR_MIN_LENGTH;
    regularMaxLength = DEFAULT_REGULAR_MAX_LENGTH;
}

Glib::ustring PCalOptions::color_to_str(const Gdk::Color& color)
{
    char buf[8];
    
    snprintf(buf,8,"#%.2X%.2X%.2X",
                    color.get_red()/256, 
                    color.get_green()/256, 
                    color.get_blue()/256);

    return Glib::ustring(buf);
}

void PCalOptions::setKey(const Glib::ustring key, const Glib::ustring value) 
{
    Gdk::Color tmpcolor;
    int tmpint;
    
    if(key=="First Day") {
        if(value=="1") firstDay = Glib::Date::MONDAY;
        else firstDay = Glib::Date::SUNDAY;
    } else
    if(key=="Toolbar Style") {
        toolbarStyle = value;
    } else
    
    // new option placeholder
    
    if(key=="Undefined Month BG") {
        if(tmpcolor.parse(value)) {
            undefinedMonthBGColor = tmpcolor;
        }
    } else
    if(key=="Undefined Month FG") {
        if(tmpcolor.parse(value)) {
            undefinedMonthFGColor = tmpcolor;
        }
    } else
    if(key=="Inactive Month BG") {
        if(tmpcolor.parse(value)) {
            inactiveMonthBGColor = tmpcolor;
        }
    } else
    if(key=="Inactive Month FG") {
        if(tmpcolor.parse(value)) {
            inactiveMonthFGColor = tmpcolor;
        }
    } else
    if(key=="Today BG") {
        if(tmpcolor.parse(value)) {
            todayBGColor = tmpcolor;
        }
    } else
    if(key=="Today FG") {
        if(tmpcolor.parse(value)) {
            todayFGColor = tmpcolor;
        }
    } else
    if(key=="Day 1 BG") {
        if(tmpcolor.parse(value)) {
            day1BGColor = tmpcolor;
        }
    } else
    if(key=="Day 1 FG") {
        if(tmpcolor.parse(value)) {
            day1FGColor = tmpcolor;
        }
    } else
    if(key=="Predicted Day 1 BG") {
        if(tmpcolor.parse(value)) {
            pday1BGColor = tmpcolor;
        }
    } else
    if(key=="Predicted Day 1 FG") {
        if(tmpcolor.parse(value)) {
            pday1FGColor = tmpcolor;
        }
    } else
    if(key=="Fertile BG") {
        if(tmpcolor.parse(value)) {
            fertBGColor = tmpcolor;
        }
    } else
    if(key=="Fertile FG") {
        if(tmpcolor.parse(value)) {
            fertFGColor = tmpcolor;
        }
    } else
    if(key=="Predicted Fertile BG") {
        if(tmpcolor.parse(value)) {
            pfertBGColor = tmpcolor;
        }
    } else
    if(key=="Predicted Fertile FG") {
        if(tmpcolor.parse(value)) {
            pfertFGColor = tmpcolor;
        }
    } else
    if(key=="Pregnancy BG") {
        if(tmpcolor.parse(value)) {
            pregBGColor = tmpcolor;
        }
    } else
    if(key=="Pregnancy FG") {
        if(tmpcolor.parse(value)) {
            pregFGColor = tmpcolor;
        }
    } else
    if(key=="Birth BG") {
        if(tmpcolor.parse(value)) {
            birthBGColor = tmpcolor;
        }
    } else
    if(key=="Birth FG") {
        if(tmpcolor.parse(value)) {
            birthFGColor = tmpcolor;
        }
    } else
    if(key=="Predicted Birth BG") {
        if(tmpcolor.parse(value)) {
            pbirthBGColor = tmpcolor;
        }
    } else
    if(key=="Predicted Birth FG") {
        if(tmpcolor.parse(value)) {
            pbirthFGColor = tmpcolor;
        }
    } else
    if(key=="Buffer Days") {        
        errno = 0;
        tmpint = atoi(value.c_str());
        if(errno!=ERANGE) bufferDays = tmpint;
    } else
    if(key=="Bad Feels Before") {
        errno = 0;
        tmpint = atoi(value.c_str());
        if(errno!=ERANGE) badFeelDaysBefore = tmpint;
    } else
    if(key=="Bad Feels After") {
        errno = 0;
        tmpint = atoi(value.c_str());
        if(errno!=ERANGE) badFeelDaysAfter = tmpint;
    } else
    if(key=="Reg. Min Length") {
        errno = 0;
        tmpint = atoi(value.c_str());
        if(errno!=ERANGE) regularMinLength = tmpint;
    } else
    if(key=="Reg. Max Length") {
        errno = 0;
        tmpint = atoi(value.c_str());
        if(errno!=ERANGE) regularMaxLength = tmpint;
    }
}
